@@ -19,6 +19,7 @@
 
 enum {
     PAGE_SELECTED,
+    SHOW_PAGE,
     LAST_SIGNAL
 };
 static guint signals[LAST_SIGNAL] = { 0, };
@@ -126,8 +127,7 @@ static void
 add_cb (Book *book, Page *page, BookView *view)
 {
     PageView *page_view;
-    page_view = page_view_new ();
-    page_view_set_page (page_view, page);
+    page_view = page_view_new (page);
     g_signal_connect (page_view, "changed", G_CALLBACK (page_view_changed_cb), view);
     g_signal_connect (page_view, "size-changed", G_CALLBACK (page_view_size_changed_cb), view);  
     g_hash_table_insert (view->priv->page_data, page, page_view);
@@ -332,14 +332,15 @@ layout_into (BookView *view, gint width, gint height, gint *book_width, gint *bo
         Page *p = page_view_get_page (page);
         gint h;
 
+        /* NOTE: Using double to avoid overflow for large images */
         if (max_aspect > aspect) {
             /* Set width scaled on DPI and maximum width */
-            gint w = page_get_width (p) * max_dpi * width / (page_get_dpi (p) * max_width);
+            gint w = (double)page_get_width (p) * max_dpi * width / (page_get_dpi (p) * max_width);
             page_view_set_width (page, w);
         }
         else {
             /* Set height scaled on DPI and maximum height */
-            gint h = page_get_height (p) * max_dpi * height / (page_get_dpi (p) * max_height);
+            gint h = (double)page_get_height (p) * max_dpi * height / (page_get_dpi (p) * max_height);
             page_view_set_height (page, h);
         }
 
@@ -452,6 +453,18 @@ expose_cb (GtkWidget *widget, GdkEventExpose *event, BookView *view)
         cairo_translate (context, -get_x_offset (view), 0);
         page_view_render (page, context);
         cairo_restore (context);
+
+        if (page_view_get_selected (page))
+            gtk_paint_focus (gtk_widget_get_style (view->priv->widget),
+                             gtk_widget_get_window (view->priv->widget),
+                             GTK_STATE_SELECTED,
+                             &event->area,
+                             NULL,
+                             NULL,
+                             page_view_get_x_offset (page) - get_x_offset (view),
+                             page_view_get_y_offset (page),
+                             page_view_get_width (page),
+                             page_view_get_height (page));
     }
 
     cairo_destroy (context);
@@ -496,7 +509,9 @@ button_cb (GtkWidget *widget, GdkEventButton *event, BookView *view)
 
     gtk_widget_grab_focus (view->priv->widget);
 
-    select_page (view, get_page_at (view, event->x + get_x_offset (view), event->y, &x, &y));
+    if (event->type == GDK_BUTTON_PRESS)
+        select_page (view, get_page_at (view, event->x + get_x_offset (view), event->y, &x, &y));
+
     if (!view->priv->selected_page)
         return FALSE;
 
@@ -504,8 +519,10 @@ button_cb (GtkWidget *widget, GdkEventButton *event, BookView *view)
     if (event->button == 1) {
         if (event->type == GDK_BUTTON_PRESS)
             page_view_button_press (view->priv->selected_page, x, y);
-        else
+        else if (event->type == GDK_BUTTON_RELEASE)
             page_view_button_release (view->priv->selected_page, x, y);
+        else if (event->type == GDK_2BUTTON_PRESS)
+            g_signal_emit (view, signals[SHOW_PAGE], 0, book_view_get_selected (view));
     }
 
     /* Show pop-up menu on right click */
@@ -702,6 +719,14 @@ book_view_class_init (BookViewClass *klass)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__POINTER,
                       G_TYPE_NONE, 1, G_TYPE_POINTER);
+    signals[SHOW_PAGE] =
+        g_signal_new ("show-page",
+                      G_TYPE_FROM_CLASS (klass),
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (BookViewClass, show_page),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__POINTER,
+                      G_TYPE_NONE, 1, G_TYPE_POINTER);
 
     g_type_class_add_private (klass, sizeof (BookViewPrivate));
 }
